"""
Initialise the views module for the application.

This module is responsible for setting up and exposing the views used in the application.
Views handle HTTP requests and return HTTP responses, typically in the form of rendered templates
or JSON data for APIs.
"""
