# Generated by Django 4.2.24 on 2025-09-23 06:15

# Django
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aasrp", "0013_setting_loss_value_source"),
    ]

    operations = [
        migrations.AlterField(
            model_name="srprequest",
            name="ship",
            field=models.PositiveBigIntegerField(
                blank=True, default=None, null=True, verbose_name="Ship type ID"
            ),
        ),
        migrations.RenameField(
            model_name="srprequest",
            old_name="ship",
            new_name="ship_id",
        ),
    ]
